__author__ = 'mnowotka'

from bottle import request, response
import base64
from chembl_beaker.beaker.utils.io import _parseFlag
from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.rasterImages.impl import _ctab2image, _smiles2image
from chembl_beaker.beaker.core_apps.rasterImages.impl import _highlightCtabFragment, _highlightSmilesFragment

# ----------------------------------------------------------------------------------------------------------------------


def ctab2imageView(data, params):

    kwargs = dict()
    kwargs['size'] = int(params.get('size', 200))
    separator = params.get('separator', '|')
    kwargs['legend'] = params.get('legend', '').split(separator)
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['atomMapNumber'] = _parseFlag(params.get('atomMapNumber', False))
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))

    response.content_type = 'image/png'
    ret = _ctab2image(data, **kwargs)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2image/<ctab>', method=['OPTIONS', 'GET'], name="ctab2image")
def ctab2image(ctab):
    """
Converts CTAB to PNG image. CTAB is urlsafe_base64 encoded string containing
single molfile or concatenation of multiple molfiles. Size is the optional size of image in pixels (default value
is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}ctab2image/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_") > aspirin.png
    curl -X GET ${BEAKER_ROOT_URL}ctab2image/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?computeCoords=0 > aspirin.png
    curl -X GET ${BEAKER_ROOT_URL}ctab2image/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?atomMapNumber=1 > aspirin.png
    curl -X GET ${BEAKER_ROOT_URL}ctab2image/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin.png
    curl -X GET "${BEAKER_ROOT_URL}ctab2image/"$(cat mcs.sdf | base64 -w 0 | tr "+/" "-_")"?legend=foo|bar|bla" > out.png
    curl -X GET "${BEAKER_ROOT_URL}ctab2image/"$(cat mcs.sdf | base64 -w 0 | tr "+/" "-_")"?legend=foo|bar|bla&computeCoords=0" > out.png
    curl -X GET "${BEAKER_ROOT_URL}ctab2image/"$(cat mcs.sdf | base64 -w 0 | tr "+/" "-_")"?legend=foo|bar|bla" > out.png
    curl -X GET ${BEAKER_ROOT_URL}ctab2image/$(cat mcs_no_coords.sdf | base64 -w 0 | tr "+/" "-_")?legend=foo > out.png
    curl -X GET ${BEAKER_ROOT_URL}ctab2image/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin.png
    """

    data = base64.urlsafe_b64decode(ctab)
    return ctab2imageView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2image', method=['OPTIONS', 'POST'], name="ctab2image")
def ctab2image():
    """
Converts CTAB to PNG image. CTAB is either single molfile or SDF file. Size is the optional size of image in pixels
(default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}ctab2image > aspirin.png
    curl -X POST -F "file=@aspirin.mol" -F "computeCoords=0" ${BEAKER_ROOT_URL}ctab2image > aspirin.png
    curl -X POST -F "file=@aspirin.mol" -F "atomMapNumber=1" ${BEAKER_ROOT_URL}ctab2image > aspirin.png
    curl -X POST -F "file=@aspirin.mol" -F "legend=aspirin" ${BEAKER_ROOT_URL}ctab2image > aspirin.png
    curl -X POST -F "file=@mcs.sdf" -F "legend=foo|bar|bla" ${BEAKER_ROOT_URL}ctab2image > out.png
    curl -X POST -F "file=@mcs.sdf" -F "legend=foo|bar|bla" -F "computeCoords=0" ${BEAKER_ROOT_URL}ctab2image > out.png
    curl -X POST -F "file=@mcs_no_coords.sdf" -F "legend=foo|bar|bla" ${BEAKER_ROOT_URL}ctab2image > out.png
    curl -X POST -F "file=@mcs.sdf" -F "legend=foo" ${BEAKER_ROOT_URL}ctab2image > out.png
    curl -X POST -F "file=@mcs.sdf" -F "legend=foo|bar|bla" -F "size=400" ${BEAKER_ROOT_URL}ctab2image > out.png
    curl -X POST -F "file=@aspirin.mol" -F "size=400" ${BEAKER_ROOT_URL}ctab2image > aspirin.png
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return ctab2imageView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def smiles2imageView(data, params):

    kwargs = dict()
    kwargs['size'] = int(params.get('size', 200))
    separator = params.get('separator', '|')
    kwargs['kekulize'] = _parseFlag(params.get('kekulize', True))
    kwargs['legend'] = params.get('legend', '').split(separator)
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['atomMapNumber'] = _parseFlag(params.get('atomMapNumber', False))

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    response.content_type = 'image/png'
    ret = _smiles2image(data, **kwargs)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2image/<smiles>', method=['OPTIONS', 'GET'], name="smiles2image")
def smiles2image(smiles):
    """
Converts SMILES to PNG image. This method accepts urlsafe_base64 encoded string containing single or multiple
SMILES optionally containing header line, specific to *.smi format. Size is the optional size of image in pixels
(default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}smiles2image/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_") > aspirin.png
    curl -X GET ${BEAKER_ROOT_URL}smiles2image/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_") > aspirin.png
    curl -X GET ${BEAKER_ROOT_URL}smiles2image/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?atomMapNumber=1 > aspirin.png
    curl -X GET ${BEAKER_ROOT_URL}smiles2image/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin.png
    curl -X GET ${BEAKER_ROOT_URL}smiles2image/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin.png
    curl -X GET ${BEAKER_ROOT_URL}smiles2image/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin.png
    curl -X GET ${BEAKER_ROOT_URL}smiles2image/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin.png
    curl -X GET "${BEAKER_ROOT_URL}smiles2image/"$(cat mcs.smi | base64 -w 0 | tr "+/" "-_")"?legend=foo|bar|bla" > out.png
    curl -X GET "${BEAKER_ROOT_URL}smiles2image/"$(cat mcs_no_header.smi | base64 -w 0 | tr "+/" "-_")"?legend=foo|bar|bla" > out.png
    curl -X GET ${BEAKER_ROOT_URL}smiles2image/$(cat mcs.smi | base64 -w 0 | tr "+/" "-_")?legend=foo > out.png
    curl -X GET ${BEAKER_ROOT_URL}smiles2image/$(cat mcs_no_header.smi | base64 -w 0 | tr "+/" "-_")?legend=foo > out.png
    """

    data = base64.urlsafe_b64decode(smiles)
    return smiles2imageView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2image', method=['OPTIONS', 'POST'], name="smiles2image")
def smiles2image():
    """
Converts SMILES to PNG image. This method accepts single or multiple SMILES or *.smi file. Size is the optional
size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X POST -F "file=@aspirin_no_header.smi" ${BEAKER_ROOT_URL}smiles2image > aspirin.png
    curl -X POST -F "file=@aspirin_no_header.smi" -F "atomMapNumber=1" ${BEAKER_ROOT_URL}smiles2image > aspirin.png
    curl -X POST --data-binary @aspirin_with_header.smi ${BEAKER_ROOT_URL}smiles2image > aspirin.png
    curl -X POST --data-binary @aspirin_no_header.smi ${BEAKER_ROOT_URL}smiles2image > aspirin.png
    curl -X POST -F "file=@aspirin_with_header.smi" -F "legend=aspirin" ${BEAKER_ROOT_URL}smiles2image > aspirin.png
    curl -X POST -F "file=@aspirin_no_header.smi" -F "legend=aspirin" ${BEAKER_ROOT_URL}smiles2image > aspirin.png
    curl -X POST -F "file=@aspirin_with_header.smi" -F "size=400" ${BEAKER_ROOT_URL}smiles2image > aspirin.png
    curl -X POST -F "file=@aspirin_no_header.smi" -F "size=400" ${BEAKER_ROOT_URL}smiles2image > aspirin.png
    curl -X POST -F "file=@mcs.smi" -F "legend=foo|bar|bla" ${BEAKER_ROOT_URL}smiles2image > out.png
    curl -X POST -F "file=@mcs_no_header.smi" -F "legend=foo|bar|bla" ${BEAKER_ROOT_URL}smiles2image > out.png
    curl -X POST -F "file=@mcs.smi" -F "legend=foo" ${BEAKER_ROOT_URL}smiles2image > out.png
    curl -X POST -F "file=@mcs_no_header.smi" -F "legend=foo" ${BEAKER_ROOT_URL}smiles2image > out.png
    curl -X POST -F "file=@mcs.smi" -F "legend=foo|bar|bla" -F "size=400" ${BEAKER_ROOT_URL}smiles2image > out.png
    curl -X POST -F "file=@mcs_no_header.smi" -F "legend=foo|bar|bla" -F "size=400" ${BEAKER_ROOT_URL}smiles2image > out.png
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return smiles2imageView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def highlightSmilesFragmentView(data, params):

    kwargs = dict()
    kwargs['size'] = int(params.get('size', 200))
    separator = params.get('separator', '|')
    smarts = params.get('smarts', '')
    kwargs['kekulize'] = _parseFlag(params.get('kekulize', True))
    kwargs['legend'] = params.get('legend', '').split(separator)
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['atomMapNumber'] = _parseFlag(params.get('atomMapNumber', False))
    kwargs['force'] = _parseFlag(params.get('force', True))

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    response.content_type = 'image/png'
    ret = _highlightSmilesFragment(data, smarts, **kwargs)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/highlightSmilesFragment/<smarts>/<smiles>', method=['OPTIONS', 'GET'], name="highlightSmilesFragment")
def highlightSmilesFragment(smarts, smiles):
    """
Converts SMILES to PNG image and highlight the fragment described by SMARTS. 
This method accepts urlsafe_base64 encoded string containing single or multiple SMILES optionally containing 
header line, specific to *.smi format. Size is the optional size of image in pixels (default value is 200 px). 
Legend is optional label in the bottom of image.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_") > aspirin_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_") > aspirin_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?atomMapNumber=1 > aspirin_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin_highlighted.png
    curl -X GET "${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/"$(cat mcs.smi | base64 -w 0 | tr "+/" "-_")"?legend=foo|bar|bla" > out_highlighted.png
    curl -X GET "${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/"$(cat mcs_no_header.smi | base64 -w 0 | tr "+/" "-_")"?legend=foo|bar|bla" > out_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat mcs.smi | base64 -w 0 | tr "+/" "-_")?legend=foo > out_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat mcs_no_header.smi | base64 -w 0 | tr "+/" "-_")?legend=foo > out_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragment/$(cat aspirin.sma | base64 -w 0 | tr "+/" "-_")/$(cat CHEMBL1999443.smi | base64 -w 0 | tr "+/" "-_")?force=true > out_highlighted_forced.png
    """

    data = base64.urlsafe_b64decode(smiles)

    params = request.params
    params['smarts'] = base64.urlsafe_b64decode(smarts)

    return highlightSmilesFragmentView(data, params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/highlightSmilesFragment', method=['OPTIONS', 'POST'], name="highlightSmilesFragment")
def highlightSmilesFragment():
    """
Converts SMILES to PNG image and highlight the fragment described by SMARTS.
This method accepts single or multiple SMILES or *.smi file. Size is the optional
size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X POST -F "file=@aspirin_no_header.smi" -F "smarts=c1ccccc1" ${BEAKER_ROOT_URL}highlightSmilesFragment > aspirin_highlighted.png
    curl -X POST -F "file=@aspirin_no_header.smi" -F "smarts=c1ccccc1" -F "atomMapNumber=1" ${BEAKER_ROOT_URL}highlightSmilesFragment > aspirin_highlighted.png
    curl -X POST -F "file=@aspirin_with_header.smi" -F "smarts=c1ccccc1" -F "legend=aspirin" ${BEAKER_ROOT_URL}highlightSmilesFragment > aspirin_highlighted.png
    curl -X POST -F "file=@aspirin_no_header.smi" -F "smarts=c1ccccc1" -F "legend=aspirin" ${BEAKER_ROOT_URL}highlightSmilesFragment > aspirin_highlighted.png
    curl -X POST -F "file=@aspirin_with_header.smi" -F "smarts=c1ccccc1" -F "size=400" ${BEAKER_ROOT_URL}highlightSmilesFragment > aspirin_highlighted.png
    curl -X POST -F "file=@aspirin_no_header.smi" -F "smarts=c1ccccc1" -F "size=400" ${BEAKER_ROOT_URL}highlightSmilesFragment > aspirin_highlighted.png
    curl -X POST -F "file=@mcs.smi" -F "smarts=c1ccccc1" -F "legend=foo|bar|bla" ${BEAKER_ROOT_URL}highlightSmilesFragment > out_highlighted.png
    curl -X POST -F "file=@mcs_no_header.smi" -F "smarts=c1ccccc1" -F "legend=foo|bar|bla" ${BEAKER_ROOT_URL}highlightSmilesFragment > out_highlighted.png
    curl -X POST -F "file=@mcs.smi" -F "smarts=c1ccccc1" -F "legend=foo" ${BEAKER_ROOT_URL}highlightSmilesFragment > out_highlighted.png
    curl -X POST -F "file=@mcs_no_header.smi" -F "smarts=c1ccccc1" -F "legend=foo" ${BEAKER_ROOT_URL}highlightSmilesFragment > out_highlighted.png
    curl -X POST -F "file=@mcs.smi" -F "smarts=c1ccccc1" -F "legend=foo|bar|bla" -F "size=400" ${BEAKER_ROOT_URL}highlightSmilesFragment > out_highlighted.png
    curl -X POST -F "file=@mcs_no_header.smi" -F "smarts=c1ccccc1" -F "legend=foo|bar|bla" -F "size=400" ${BEAKER_ROOT_URL}highlightSmilesFragment > out_highlighted.png
    curl -X POST -F "file=@CHEMBL1999443.smi" -F "smarts=[#6]1:[#6]:[#6]:[#6]:[#6](:[#6]:1-[#6](-[#8])=[#8])-[#8]-[#6](-[#6])=[#8]" -F "force=true" ${BEAKER_ROOT_URL}highlightSmilesFragment > out_highlighted_forced.png
    curl -X POST -F "file=@CHEMBL1999443.smi" -F "smarts=@aspirin.sma" -F "force=true" ${BEAKER_ROOT_URL}highlightSmilesFragment > out_highlighted_forced.png
    """

    number_of_files = len(request.files)
    data = None
    if number_of_files:
        if number_of_files == 1:
            data = request.files.values()[0].file.read()
        elif number_of_files == 2:
            data = request.files['file'].file.read()
            smarts = request.files['smarts'].file.read()
            request.params['smarts'] = smarts
    else:
        data = request.body.read()
    return highlightSmilesFragmentView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def highlightCtabFragmentView(data, params):

    kwargs = dict()
    kwargs['size'] = int(params.get('size', 200))
    separator = params.get('separator', '|')
    smarts = params.get('smarts', '')
    kwargs['kekulize'] = _parseFlag(params.get('kekulize', True))
    kwargs['legend'] = params.get('legend', '').split(separator)
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['atomMapNumber'] = _parseFlag(params.get('atomMapNumber', False))
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))
    kwargs['force'] = _parseFlag(params.get('force', True))

    response.content_type = 'image/png'
    ret = _highlightCtabFragment(data, smarts, **kwargs)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/highlightCtabFragment/<smarts>/<ctab>', method=['OPTIONS', 'GET'], name="highlightCtabFragment")
def highlightCtabFragment(smarts, ctab):
    """
Converts CTAB to PNG image highlighting the fragment described by SMARTS. 
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles. 
Size is the optional size of image in pixels (default value is 200 px). 
Legend is optional label in the bottom of image.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_") > out_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?computeCoords=0 > aspirin_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?atomMapNumber=1 > aspirin_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat mcs.sdf | base64 -w 0 | tr "+/" "-_")"?legend=foo|bar|bla" > out_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat mcs.sdf | base64 -w 0 | tr "+/" "-_")"?legend=foo|bar|bla&computeCoords=0" > out_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragment/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?size=500 > out_highlighted.png
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragment/$(cat aspirin.sma | base64 -w 0 | tr "+/" "-_")/$(cat CHEMBL1999443.mol | base64 -w 0 | tr "+/" "-_")?force=true > out_highlighted_forced.png
    """

    data = base64.urlsafe_b64decode(ctab)

    params = request.params
    params['smarts'] = base64.urlsafe_b64decode(smarts)

    return highlightCtabFragmentView(data, params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/highlightCtabFragment', method=['OPTIONS', 'POST'], name="highlightCtabFragment")
def highlightCtabFragment():
    """
Converts CTAB to PNG image highlighting the fragment described by SMARTS. 
CTAB is either single molfile or SDF file. Size is the optional size of image in pixels
(default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" ${BEAKER_ROOT_URL}highlightCtabFragment > aspirin_highlighted.png
    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" -F "computeCoords=0" ${BEAKER_ROOT_URL}highlightCtabFragment > aspirin_highlighted.png
    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" -F "atomMapNumber=1" ${BEAKER_ROOT_URL}highlightCtabFragment > aspirin_highlighted.png
    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" -F "legend=aspirin" ${BEAKER_ROOT_URL}highlightCtabFragment > aspirin_highlighted.png
    curl -X POST -F "file=@mcs.sdf" -F "smarts=c1ccccc1" -F "legend=foo|bar|bla" ${BEAKER_ROOT_URL}highlightCtabFragment > out_highlighted.png
    curl -X POST -F "file=@mcs.sdf" -F "smarts=c1ccccc1" -F "legend=foo|bar|bla" -F "computeCoords=0" ${BEAKER_ROOT_URL}highlightCtabFragment > out_highlighted.png
    curl -X POST -F "file=@mcs_no_coords.sdf" -F "smarts=c1ccccc1" -F "legend=foo|bar|bla" ${BEAKER_ROOT_URL}highlightCtabFragment > out_highlighted.png
    curl -X POST -F "file=@mcs.sdf" -F "smarts=c1ccccc1" -F "legend=foo" ${BEAKER_ROOT_URL}highlightCtabFragment > out_highlighted.png
    curl -X POST -F "file=@mcs.sdf" -F "smarts=c1ccccc1" -F "legend=foo|bar|bla" -F "size=400" ${BEAKER_ROOT_URL}highlightCtabFragment > out_highlighted.png
    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" -F "size=400" ${BEAKER_ROOT_URL}highlightCtabFragment > aspirin_highlighted.png
    curl -X POST -F "file=@CHEMBL1999443.mol" -F "smarts=[#6]1:[#6]:[#6]:[#6]:[#6](:[#6]:1-[#6](-[#8])=[#8])-[#8]-[#6](-[#6])=[#8]" -F "force=true" ${BEAKER_ROOT_URL}highlightCtabFragment > out_highlighted_forced.png
    curl -X POST -F "file=@CHEMBL1999443.mol" -F "smarts=@aspirin.sma" -F "force=true" ${BEAKER_ROOT_URL}highlightCtabFragment > out_highlighted_forced.png    
    """

    number_of_files = len(request.files)
    data = None
    if number_of_files:
        if number_of_files == 1:
            data = request.files.values()[0].file.read()
        elif number_of_files == 2:
            data = request.files['file'].file.read()
            smarts = request.files['smarts'].file.read()
            request.params['smarts'] = smarts
    else:
        data = request.body.read()
    return highlightCtabFragmentView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------



# $Id$
#
#  Copyright (C) 2013 Michal Nowotka and Greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#

import re
import json
from rdkit import Chem
from rdkit.Chem.Draw.canvasbase import CanvasBase
from chembl_beaker.beaker.draw.molDrawing import MolDrawing

try:
    from chembl_beaker.beaker.draw.molDrawing import DrawingOptions
except ImportError:
    DrawingOptions = None
from rdkit.Chem.AllChem import Compute2DCoords


# ----------------------------------------------------------------------------------------------------------------------


class Canvas(CanvasBase):
    """
  The output of this can be inserted in a web page that has raphael loaded as follows:

'''
<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8">
        <title>Raphael Molecules demo</title>
        <link rel="stylesheet" href="demo.css" type="text/css" media="screen">
        <link rel="stylesheet" href="demo-print.css" type="text/css" media="print">
        <script src="raphael-min.js"></script>
        <style media="screen">
            body {
                margin: 0;
                padding: 0;
                text-align: center;
            }
            h1 {
                font-weight: 400;
                height: 5%%;
            }
            #canvas {
                height: 480px;
                margin: 0 auto;
                text-align: left;
                width: 640px;
            }
            #code {
                font-family: Consolas, Monaco, "Lucida Console", monospace;
                height: 4em;
                margin: 10px;
                padding: 0;
                width: 90%%;
            }
            #run {
                font-size: 2em;
            }
        </style>
        <script>
            window.onload = function () {
                var paper = Raphael("canvas", 600, 600);
                paper.add(%(JSON)s);
            };
        </script>
    </head>
    <body>
        <h1>Raphael Molecule Demo</h1>
        <div id="canvas"></div>
    </body>
</html>
'''

  """

# ----------------------------------------------------------------------------------------------------------------------

    def __init__(self, size=None):
        self.json = ''
        self.objects = []
        self.size = size
        self.scale = 1

# ----------------------------------------------------------------------------------------------------------------------

    def flush(self):
        self.json = json.dumps(self.objects)

# ----------------------------------------------------------------------------------------------------------------------

    def _addCanvasLine(self, p1, p2, color, **kwargs):
        line = dict()
        line['type'] = 'path'
        if kwargs.get('dash', (0, 0)) == (0, 0):
            line['path'] = "M%s,%sL%s,%s" % (p1[0], p1[1], p2[0], p2[1])
        else:
            dash = kwargs['dash']
            pts = self._getLinePoints(p1, p2, dash)
            currDash = 0
            dashOn = True
            path = ''
            while currDash < (len(pts) - 1):
                if dashOn:
                    p1 = pts[currDash]
                    p2 = pts[currDash + 1]
                    path += "M%s,%sL%s,%s" % (p1[0], p1[1], p2[0], p2[1])
                currDash += 1
                dashOn = not dashOn
            line['path'] = path

        line['stroke'] = 'rgb' + str(color)
        line['stroke-width'] = self.scale * kwargs.get('linewidth', 1)
        return line

# ----------------------------------------------------------------------------------------------------------------------

    def addCanvasLine(self, p1, p2, color=(0, 0, 0), color2=None, **kwargs):
        color = tuple(map(lambda x: x * 255, color))
        color2 = tuple(map(lambda x: x * 255, color2))
        if color2 and color2 != color:
            mp = (p1[0] + p2[0]) / 2., (p1[1] + p2[1]) / 2.
            line1 = self._addCanvasLine(p1, mp, color, **kwargs)
            line2 = self._addCanvasLine(mp, p2, color2, **kwargs)
            self.objects.append(line1)
            self.objects.append(line2)
        else:
            line = self._addCanvasLine(p1, p2, color, **kwargs)
            self.objects.append(line)

# ----------------------------------------------------------------------------------------------------------------------

    def addCanvasText(self, text, pos, font, color=(0, 0, 0), **kwargs):
        color = tuple(map(lambda x: x * 255, color))
        bgColor = tuple(map(lambda x: x * 255, kwargs.get('bgColor', (1, 1, 1))))

        orientation = kwargs.get('orientation', 'E')
        text = re.sub(r'\<.+?\>', '', text)
        w, h = font.size * len(text), font.size
        offset = w * pos[2]
        bw, bh = w, h * 1.4
        xp, yp = pos[0], pos[1]
        if orientation == 'E':
            xp += w / 2
        elif orientation == 'W':
            xp -= w / 2
        tex = dict()
        tex['type'] = 'text'
        tex['x'] = xp
        tex['y'] = yp
        tex['text'] = text
        tex['font-size'] = font.size
        tex['stroke'] = 'rgb' + str(color)
        if font.weight == 'bold':
            tex['font-weight'] = "bold"
        backgroundRect = dict()
        backgroundRect['type'] = 'rect'

        if bgColor is not None:
            backgroundRect['x'] = xp - w / 2
            backgroundRect['y'] = yp - h / 2
            backgroundRect['width'] = w
            backgroundRect['height'] = h
            backgroundRect['fill'] = 'rgb' + str(bgColor)
            backgroundRect['stroke'] = 'rgb' + str(bgColor)
            self.objects.append(backgroundRect)
        self.objects.append(tex)
        return bw, bh, offset

# ----------------------------------------------------------------------------------------------------------------------

    def addCanvasPolygon(self, ps, color=(0, 0, 0), fill=True, stroke=False, **kwargs):
        if not fill and not stroke:
            return
        color = tuple(map(lambda x: x * 255, color))
        polygon = dict()
        polygon['type'] = 'path'
        path = ''
        path += "M%s,%s" % (ps[0][0], ps[0][1])
        for p in ps[1:]:
            path += "L%s,%s" % (p[0], p[1])
        path += 'Z'
        polygon['path'] = path

        if fill:
            polygon['fill'] = 'rgb' + str(color)
        if stroke:
            polygon['stroke'] = 'rgb' + str(color)
        self.objects.append(polygon)

# ----------------------------------------------------------------------------------------------------------------------

    def addCanvasDashedWedge(self, p1, p2, p3, dash=(2, 2), color=(0, 0, 0), color2=None, **kwargs):
        color = tuple(map(lambda x: x * 255, color))
        wedge = dict()
        wedge['stroke-width'] = self.scale * kwargs.get('linewidth', 1)
        wedge['stroke'] = 'rgb' + str(color)
        wedge['type'] = 'path'

        dash = (3, 3)
        pts1 = self._getLinePoints(p1, p2, dash)
        pts2 = self._getLinePoints(p1, p3, dash)

        if len(pts2) < len(pts1):
            pts2, pts1 = pts1, pts2

        path = ''
        for i in range(len(pts1)):
            path += "M%s,%s" % (pts1[i][0], pts1[i][1])
            path += "L%s,%s" % (pts2[i][0], pts2[i][1])

        wedge['path'] = path
        self.objects.append(wedge)

# ----------------------------------------------------------------------------------------------------------------------

    def addCircle(self, center, radius, color=(0, 0, 0), fill=True, stroke=False, alpha=1.0, **kwargs):
        if not fill and not stroke:
            return
        color = tuple(map(lambda x: x * 255, color))
        circle = dict()
        circle['type'] = 'circle'
        circle['cx'] = center[0]
        circle['cy'] = center[1]
        circle['r'] = radius
        if fill:
            circle['fill'] = 'rgb' + str(color)
        if stroke:
            circle['stroke'] = 'rgb' + str(color)
        if alpha:
            circle['opacity'] = alpha
        self.objects.append(circle)


# ----------------------------------------------------------------------------------------------------------------------


def MolToJSON(mol, size=(300, 300), kekulize=True, wedgeBonds=True, fitImage=False, options=None, **kwargs):
    if not mol:
        raise ValueError, 'Null molecule provided'

    canvas = Canvas(size=size)

    if options is None:
        options = DrawingOptions()
    if fitImage:
        options.dotsPerAngstrom = int(min(size) / 10)
    options.wedgeDashedBonds = wedgeBonds
    try:
        drawer = MolDrawing(canvas=canvas, drawingOptions=options)
    except TypeError:
        drawer = MolDrawing(canvas=canvas)

    if kekulize:
        mol = Chem.Mol(mol.ToBinary())
        Chem.Kekulize(mol)

    if not mol.GetNumConformers():
        Compute2DCoords(mol)

    drawer.AddMol(mol, **kwargs)
    try:
        drawer.AddLegend(kwargs.get('legend', ''))
    except AttributeError:
        pass

    canvas.flush()
    return canvas.json

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

from rdkit import Chem
try:
    from rdkit.Chem import rdFMCS as MCS
except:
    from rdkit.Chem import MCS
from chembl_beaker.beaker.utils.io import _parseMolData


# ----------------------------------------------------------------------------------------------------------------------


def _mcs(data, asSmiles, atomCompare, bondCompare, threshold, ringMatchesRingOnly, completeRingsOnly, sanitize=True,
         removeHs=True, strictParsing=True, isomericSmiles=False, canonical=True, kekuleSmiles=False):
    ms = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    if not ms:
        return
    if len(ms) == 1:
        if asSmiles:
            return Chem.MolToSmiles(ms[0])
        else:
            return Chem.MolToSmarts(ms[0])

    if threshold:
        threshold = float(threshold)
    try:
        mcs = MCS.FindMCS(ms,
                          atomCompare=atomCompare,
                          bondCompare=bondCompare,
                          ringMatchesRingOnly=ringMatchesRingOnly,
                          completeRingsOnly=completeRingsOnly,
                          threshold=threshold)
    except TypeError:
        ac = MCS.AtomCompare.CompareAny
        if hasattr(MCS.AtomCompare, atomCompare):
            ac = getattr(MCS.AtomCompare, atomCompare)
        bc = MCS.BondCompare.CompareOrder
        if hasattr(MCS.BondCompare, bondCompare):
            bc = getattr(MCS.BondCompare, bondCompare)
        th = 1.0
        if threshold:
            th = threshold
        mcs = MCS.FindMCS(ms,
                          atomCompare=ac,
                          bondCompare=bc,
                          ringMatchesRingOnly=ringMatchesRingOnly,
                          completeRingsOnly=completeRingsOnly,
                          threshold=th
                          )
    if hasattr(mcs, 'smarts'):
        res = mcs.smarts
    else:
        res = mcs.smartsString
    if asSmiles:
        p = Chem.MolFromSmarts(res)
        for m in ms:
            if m.HasSubstructMatch(p):
                match = m.GetSubstructMatch(p)
                res = Chem.MolFragmentToSmiles(m, atomsToUse=match, isomericSmiles=isomericSmiles, canonical=canonical,
                                               kekuleSmiles=kekuleSmiles)
                break
    return res

# ----------------------------------------------------------------------------------------------------------------------




__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

import os
import tempfile
import StringIO
from rdkit.Chem import SDMolSupplier
from rdkit.Chem import SmilesMolSupplier
from rdkit.Chem import MolToSmarts
from rdkit.Chem import MolFromSmarts
from rdkit.Chem import SDWriter
from rdkit.Chem import SmilesWriter
from chembl_beaker.beaker.utils.functional import _apply, _call
from chembl_beaker.beaker.utils.chemical_transformation import _computeCoords
from chembl_beaker.beaker.utils.chemical_transformation import _getSubstructMatch

# ----------------------------------------------------------------------------------------------------------------------


def _parseFlag(data):
    try:
        return bool(int(data))
    except ValueError:
        if isinstance(data, basestring):
            l = data.lower()
            if l == 't' or l == 'true':
                return True
            if l == 'f' or l == 'false':
                return False
        if not data:
            return False
        return None

# ----------------------------------------------------------------------------------------------------------------------


def _parseMolData(data, sanitize=True, removeHs=True, strictParsing=True):
    fd, fpath = tempfile.mkstemp(text=True)
    os.write(fd, data)
    os.close(fd)
    suppl = SDMolSupplier(fpath, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    res = [x for x in suppl if x]
    os.remove(fpath)
    return res

# ----------------------------------------------------------------------------------------------------------------------


def _parseSMILESData(data, computeCoords=False, delimiter=' ', smilesColumn=0, nameColumn=1, titleLine=True,
                     sanitize=True):
    fd, fpath = tempfile.mkstemp(text=True)
    os.write(fd, data)
    os.close(fd)
    suppl = SmilesMolSupplier(fpath, delimiter=delimiter, smilesColumn=smilesColumn, nameColumn=nameColumn,
                              titleLine=titleLine, sanitize=sanitize)
    mols = [x for x in suppl if x]
#    if not mols:
#        mols = [MolFromSmiles(data, sanitize=sanitize)]
    if computeCoords:
        _apply(mols, _computeCoords, True)
    _call(mols, 'SetProp', '_Name', '')
    os.remove(fpath)
    return mols

# ----------------------------------------------------------------------------------------------------------------------


def _getSDFStream(f, mols):
    w = SDWriter(f)
    for m in mols:
        w.write(m)
    w.flush()

# ----------------------------------------------------------------------------------------------------------------------


def _getSDFString(mols):
    sio = StringIO.StringIO()
    _getSDFStream(sio, mols)
    return sio.getvalue()

# ----------------------------------------------------------------------------------------------------------------------


def _getSMILESStream(f, mols, delimiter=' ', nameHeader='Name', includeHeader=True, isomericSmiles=False,
                     kekuleSmiles=False):
    w = SmilesWriter(f, delimiter=delimiter, nameHeader=nameHeader, includeHeader=includeHeader,
                     isomericSmiles=isomericSmiles, kekuleSmiles=kekuleSmiles)
    for mol in mols:
        w.write(mol)
    w.flush()

# ----------------------------------------------------------------------------------------------------------------------


def _getSMILESString(mols, delimiter=' ', nameHeader='Name', includeHeader=True, isomericSmiles=False,
                     kekuleSmiles=False):
    sio = StringIO.StringIO()
    _getSMILESStream(sio, mols, delimiter=delimiter, nameHeader=nameHeader, includeHeader=includeHeader,
                     isomericSmiles=isomericSmiles, kekuleSmiles=kekuleSmiles)
    return sio.getvalue()

# ----------------------------------------------------------------------------------------------------------------------


def _getSMARTSString(mols, isomericSmiles=False):
    sio = StringIO.StringIO()
    for mol in mols:
        sio.write(MolToSmarts(mol, isomericSmiles) + '\n')
    return sio.getvalue()

# ----------------------------------------------------------------------------------------------------------------------


def _molFromSmarts(smarts):
    return MolFromSmarts(str(smarts))

# ----------------------------------------------------------------------------------------------------------------------


def _getXYZ(mols, computeCoords=True):

    sio = StringIO.StringIO()

    for i, mol in enumerate(mols):

        molH = mol
        if computeCoords:
            from chembl_beaker.beaker.core_apps.D3Coords.impl import _2D23D
            molH = _2D23D(mol, None)

        atoms = molH.GetAtoms()
        sio.write(str(len(atoms)) + '\n')
        sio.write('\n')

        i = 0
        for conf in molH.GetConformers():

            for j in range(0, conf.GetNumAtoms()):

                sio.write(atoms[i].GetSymbol() +
                          '\t' + str(conf.GetAtomPosition(j).x) +
                          '\t' + str(conf.GetAtomPosition(j).y) +
                          '\t' + str(conf.GetAtomPosition(j).z)
                          + '\n')
                i += 1

    return sio.getvalue()

# ----------------------------------------------------------------------------------------------------------------------


def _getMatches(mols, smarts, force=False):
    _call(mols, 'UpdatePropertyCache', strict=False)
    return _apply(mols, _getSubstructMatch, _molFromSmarts(smarts), force)

# ----------------------------------------------------------------------------------------------------------------------




__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker.utils.io import _parseFlag
from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.similarityMaps.impl import _smiles2SimilarityMap, _sdf2SimilarityMap
from bottle import request, response
import base64

# ----------------------------------------------------------------------------------------------------------------------


def smiles2SimilarityMapView(data, params):

    kwargs = dict()
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', False))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['width'] = int(params.get('width', 500))
    kwargs['height'] = int(params.get('height', 500))
    kwargs['radius'] = int(params.get('radius', 2))
    kwargs['fingerprint'] = params.get('fingerprint', 'morgan')
    kwargs['format'] = params.get('format', 'png')

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    response.content_type = 'image/png'
    ret = _smiles2SimilarityMap(data, **kwargs)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2SimilarityMap/<smiles>', method=['OPTIONS', 'GET'], name="smiles2SimilarityMap")
def smiles2SimilarityMap(smiles):
    """
Generates similarity map, which is a way to visualize the atomic contributions to the similarity between molecules.
This method requires *exactly two SMILES*
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}smiles2SimilarityMap/$(cat sim.smi | base64 -w 0 | tr "+/" "-_") > sim.png
    curl -X GET "${BEAKER_ROOT_URL}smiles2SimilarityMap/"$(cat sim.smi | base64 -w 0 | tr "+/" "-_")"?width=500&height=500" > sim.png
    curl -X GET "${BEAKER_ROOT_URL}smiles2SimilarityMap/"$(cat sim.smi | base64 -w 0 | tr "+/" "-_")"?width=500&height=500&fingerprint=tt" > sim.png
    curl -X GET "${BEAKER_ROOT_URL}smiles2SimilarityMap/"$(cat sim.smi | base64 -w 0 | tr "+/" "-_")"?width=500&height=500&fingerprint=ap" > sim.png
    curl -X GET "${BEAKER_ROOT_URL}smiles2SimilarityMap/"$(cat sim.smi | base64 -w 0 | tr "+/" "-_")"?format=svg" > sim.svg

    """

    data = base64.urlsafe_b64decode(smiles)
    return smiles2SimilarityMapView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2SimilarityMap', method=['OPTIONS', 'POST'], name="smiles2SimilarityMap")
def smiles2SimilarityMap():
    """
Generates similarity map, which is a way to visualize the atomic contributions to the similarity between molecules.
This method requires *exactly two SMILES*
cURL examples:

    curl -X POST --data-binary @sim.smi ${BEAKER_ROOT_URL}smiles2SimilarityMap > sim.png
    curl -X POST -F "file=@sim.smi" -F "width=500" -F "height=500" -F "fingerprint=tt" ${BEAKER_ROOT_URL}smiles2SimilarityMap > sim.png
    curl -X POST -F "file=@sim.smi" -F "width=500" -F "height=500" -F "fingerprint=ap" ${BEAKER_ROOT_URL}smiles2SimilarityMap > sim.png
    curl -X POST -F "file=@sim.smi" -F "format=svg" ${BEAKER_ROOT_URL}smiles2SimilarityMap > sim.svg
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return smiles2SimilarityMapView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def sdf2SimilarityMapView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['width'] = int(params.get('width', 500))
    kwargs['height'] = int(params.get('height', 500))
    kwargs['radius'] = int(params.get('radius', 2))
    kwargs['fingerprint'] = params.get('fingerprint', 'morgan')
    kwargs['format'] = params.get('format', 'png')

    response.content_type = 'image/png'
    ret = _sdf2SimilarityMap(data, **kwargs)
    if request.is_ajax:
        ret = base64.b64encode(ret)
    return ret

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/sdf2SimilarityMap/<ctab>', method=['OPTIONS', 'GET'], name="sdf2SimilarityMap")
def sdf2SimilarityMap(ctab):
    """
Generates similarity map, which is a way to visualize the atomic contributions to the similarity between molecules.
This method requires SDF containing *exactly two mols*
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}sdf2SimilarityMap/$(cat sim.sdf | base64 -w 0 | tr "+/" "-_") > sim.png
    curl -X GET "${BEAKER_ROOT_URL}sdf2SimilarityMap/"$(cat sim.sdf | base64 -w 0 | tr "+/" "-_")"?width=500&height=500" > sim.png
    curl -X GET "${BEAKER_ROOT_URL}sdf2SimilarityMap/"$(cat sim.sdf | base64 -w 0 | tr "+/" "-_")"?width=500&height=500&fingerprint=tt" > sim.png
    curl -X GET "${BEAKER_ROOT_URL}sdf2SimilarityMap/"$(cat sim.sdf | base64 -w 0 | tr "+/" "-_")"?width=500&height=500&fingerprint=ap" > sim.png
    curl -X GET "${BEAKER_ROOT_URL}sdf2SimilarityMap/"$(cat sim.sdf | base64 -w 0 | tr "+/" "-_")"?format=svg" > sim.svg
    """

    data = base64.urlsafe_b64decode(ctab)
    return sdf2SimilarityMapView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/sdf2SimilarityMap', method=['OPTIONS', 'POST'], name="sdf2SimilarityMap")
def sdf2SimilarityMap():
    """
Generates similarity map, which is a way to visualize the atomic contributions to the similarity between molecules.
This method requires SDF containing *exactly two mols*
cURL examples:

    curl -X POST --data-binary @sim.sdf ${BEAKER_ROOT_URL}sdf2SimilarityMap > sim.png
    curl -X POST -F "file=@sim.sdf" -F "width=500" -F "height=500" -F "fingerprint=tt" ${BEAKER_ROOT_URL}sdf2SimilarityMap > sim.png
    curl -X POST -F "file=@sim.sdf" -F "width=500" -F "height=500" -F "fingerprint=ap" ${BEAKER_ROOT_URL}sdf2SimilarityMap > sim.png
    curl -X POST -F "file=@sim.sdf" -F "width=500" -F "format=svg" ${BEAKER_ROOT_URL}sdf2SimilarityMap > sim.svg
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return sdf2SimilarityMapView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------




__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

try:
    import cairo
    cffi = False
except ImportError:
    import cairocffi
    cairocffi.install_as_pycairo()
    cffi = True
    import io
    import cairo
    if not hasattr(cairo, 'HAS_PDF_SURFACE'):
        cairo.HAS_PDF_SURFACE = False
    if not hasattr(cairo, 'HAS_SVG_SURFACE'):
        cairo.HAS_SVG_SURFACE = True



from rdkit import Chem
from chembl_beaker.beaker.utils.functional import _apply
from chembl_beaker.beaker.utils.io import _parseMolData, _parseSMILESData
from chembl_beaker.beaker.utils.chemical_transformation import _computeCoords, _atomMapNumber, _kekulize
from chembl_beaker.beaker.utils.io import _getMatches

NEW_RENDER_ENGINE = False

try:
    from rdkit.Chem.Draw import rdMolDraw2D
    NEW_RENDER_ENGINE = True
except:
    import StringIO
    from chembl_beaker.beaker.draw import cairoCanvas
    from chembl_beaker.beaker import draw

# ----------------------------------------------------------------------------------------------------------------------


def _mols2svg(mols, size, legend, kekulize=True, wedgeBonds=True, fitImage=True, atomMapNumber=False,
              computeCoords=False, highlightAtomLists=None):

    if computeCoords:
        _apply(mols, _computeCoords, True)

    if atomMapNumber:
        _apply(mols, _atomMapNumber)

    if NEW_RENDER_ENGINE:
        if kekulize:
            _apply(mols, _kekulize)

        hl = None
        if highlightAtomLists:
            hl = highlightAtomLists[0]

        drawer = rdMolDraw2D.MolDraw2DSVG(size, size)
        drawer.DrawMolecule(mols[0], highlightAtoms=hl, legend=legend)
        drawer.FinishDrawing()
        return drawer.GetDrawingText()

    molsPerRow = min(len(mols), 4)
    totalWidth = molsPerRow * size
    totalHeight = molsPerRow * size
    if cffi and cairocffi.version <= (1, 10, 0):
        imageData = io.BytesIO()
    else:
        imageData = StringIO.StringIO()
    surf = cairo.SVGSurface(imageData, totalWidth, totalHeight)
    ctx = cairo.Context(surf)
    for i, mol in enumerate(mols):
        highlight = []
        if highlightAtomLists:
            highlight = highlightAtomLists[i] or []
        tx = size*(i % molsPerRow)
        ty = size*(i // molsPerRow)
        ctx.translate(tx, ty)
        canv = cairoCanvas.Canvas(ctx=ctx, size=(size, size), imageType='svg')
        leg = mol.GetProp("_Name") if (mol.HasProp("_Name") and mol.GetProp("_Name")) else legend
        draw.MolToImage(mol, size=(size, size), legend=leg, canvas=canv, kekulize=kekulize, wedgeBonds=wedgeBonds,
                        fitImage=fitImage, highlightAtoms=highlight)
        canv.flush()
        ctx.translate(-tx, -ty)
    surf.finish()
    return imageData.getvalue()

# ----------------------------------------------------------------------------------------------------------------------


def _ctab2svg(data, size, legend, sanitize=True, removeHs=True, strictParsing=True, kekulize=True, wedgeBonds=True,
              fitImage=True, atomMapNumber=False, computeCoords=False):
    return _mols2svg(_parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing),
        size, legend, kekulize=kekulize, wedgeBonds=wedgeBonds, fitImage=fitImage, atomMapNumber=atomMapNumber,
        computeCoords=computeCoords)

# ----------------------------------------------------------------------------------------------------------------------


def _smiles2svg(data, size, legend, computeCoords=False, delimiter=' ', smilesColumn=0, nameColumn=1,
                titleLine=True, sanitize=True, kekulize=True, wedgeBonds=True, fitImage=True, atomMapNumber=False):
    return _mols2svg(_parseSMILESData(data, computeCoords=computeCoords, delimiter=delimiter,
        smilesColumn=smilesColumn, nameColumn=nameColumn, titleLine=titleLine, sanitize=sanitize), size, legend,
        kekulize=kekulize, wedgeBonds=wedgeBonds, fitImage=fitImage, atomMapNumber=atomMapNumber,
        computeCoords=computeCoords)

# ----------------------------------------------------------------------------------------------------------------------


def _inchi2svg(inchis,size,legend, kekulize=True, wedgeBonds=True, fitImage=True, atomMapNumber=False,
               computeCoords=False):
    mols = _apply(inchis.split(), Chem.MolFromInchi)
    _apply(mols, _computeCoords)
    return _mols2svg(mols, size, legend, kekulize=kekulize, wedgeBonds=wedgeBonds, fitImage=fitImage,
                        atomMapNumber=atomMapNumber, computeCoords=computeCoords)

# ----------------------------------------------------------------------------------------------------------------------


def _highlightSmilesFragmentSVG(data, smarts, size, legend, computeCoords=False, delimiter=' ', smilesColumn=0,
                                nameColumn=1, titleLine=True, sanitize=True, kekulize=True, wedgeBonds=True,
                                fitImage=True, atomMapNumber=False, force=False):
    mols = _parseSMILESData(data, computeCoords=computeCoords, delimiter=delimiter,
                            smilesColumn=smilesColumn, nameColumn=nameColumn, titleLine=titleLine, sanitize=sanitize)
    matches = _getMatches(mols, smarts, force)
    return _mols2svg(mols, size, legend, kekulize=kekulize, wedgeBonds=wedgeBonds, fitImage=fitImage,
                     atomMapNumber=atomMapNumber, computeCoords=computeCoords, highlightAtomLists=matches)

# ----------------------------------------------------------------------------------------------------------------------


def _highlightCtabFragmentSVG(data, smarts, size, legend, sanitize=True, removeHs=True, strictParsing=True,
                              kekulize=True, wedgeBonds=True, fitImage=True, atomMapNumber=False, computeCoords=False,
                              force=False):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    matches = _getMatches(mols, smarts, force)
    return _mols2svg(mols, size, legend, kekulize=kekulize, wedgeBonds=wedgeBonds, fitImage=fitImage,
                     atomMapNumber=atomMapNumber, computeCoords=computeCoords, highlightAtomLists=matches)

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker.utils.io import _parseFlag
from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.fingerprints.impl import _sdf2fps
from bottle import request
import base64
import json

#-----------------------------------------------------------------------------------------------------------------------

def sdf2fpsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['type'] = params.get('type', 'morgan')
    kwargs['radius'] = int(params.get('radius', 2))
    kwargs['n_bits'] = int(params.get('n_bits', 2048))

    return _sdf2fps(data, **kwargs)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/sdf2fps/<ctab>', method=['OPTIONS', 'GET'], name="sdf2fps")
def sdf2fps(ctab):
    """
Computes fingerprints for given compounds and writes them as fps format. Type is a optional type of fingerprints,
can be 'morgan', 'pair' or 'maccs' (default to 'morgan'). CTAB is urlsafe_base64 encoded string containing single
molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}sdf2fps/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    curl -X GET "${BEAKER_ROOT_URL}sdf2fps/"$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")"?n_bits=1024&radius=5"
    curl -X GET "${BEAKER_ROOT_URL}sdf2fps/"$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")"?type=pair"
    curl -X GET "${BEAKER_ROOT_URL}sdf2fps/"$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")"?type=maccs"

    """

    data = base64.urlsafe_b64decode(ctab)
    return sdf2fpsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/sdf2fps', method=['OPTIONS', 'POST'], name="sdf2fps")
def sdf2fps():
    """
Computes fingerprints for given compounds and writes them as fps format. Type is a optional type of fingerprints,
can be 'morgan', 'pair' or 'maccs' (default to 'morgan'). CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}sdf2fps
    curl -X POST -F "file=@aspirin.mol" -F "n_bits=1024" -F "radius=5" ${BEAKER_ROOT_URL}sdf2fps
    curl -X POST -F "file=@aspirin.mol" -F "type=maccs" ${BEAKER_ROOT_URL}sdf2fps

    """
    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return sdf2fpsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------



#!/usr/bin/env python

__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

import json
from bottle import run
from optparse import OptionParser
from chembl_beaker.beaker import app, config, loadPlugins, loadApps

# ----------------------------------------------------------------------------------------------------------------------


def main(conf_path=None):

    standalone = False

    if conf_path:
        config.load_config(conf_path)
    else:
        standalone = True
        parser = OptionParser()
        parser.add_option("-c", "--config", dest="config_path", help="path to config file", default="beaker.conf")
        (options, args) = parser.parse_args()
        conf_path = options.config_path
        config.load_config(conf_path)

    apps = json.loads(config.get('installed_apps', '[]'))
    plugins = json.loads(config.get('plugins', '[]'))

    loadApps(apps)
    loadPlugins(app, plugins)

    server = config.get('server_middleware', 'tornado')
    kwargs = {}
    if server is 'gunicorn':
        try:
            kwargs['workers'] = int(config.get('workers', '4'))
        except Exception as e:
            print e
            kwargs['workers'] = 4

    if standalone:
        run(app=app, host=config.get('bottle_host', 'localhost'), port=config.get('bottle_port', '8080'),
            debug=config.get('debug', True), server=server, **kwargs)
    else:
        return app

if __name__ == "__main__":
    main()

else:
    application = app

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

from chembl_beaker.beaker import app
from bottle import request
from chembl_beaker.beaker.core_apps.ringInfo.impl import _isAtomInRing, _atomRings
from chembl_beaker.beaker.core_apps.ringInfo.impl import _bondRings, _isBondInRing
from chembl_beaker.beaker.core_apps.ringInfo.impl import _numAtomRings, _numBondRings, _numRings
from chembl_beaker.beaker.utils.io import _parseFlag
import base64
import json

#-----------------------------------------------------------------------------------------------------------------------

def atomRingsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return json.dumps(_atomRings(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/atomRings/<ctab>', method=['OPTIONS', 'GET'], name="atomRings")
def atomRings(ctab):
    """
Returns atom indexes group by SSSR rings. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}atomRings/$(cat rings.mol | base64 -w 0 | tr "+/" "-_")

    """

    data = base64.urlsafe_b64decode(ctab)
    return atomRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/atomRings', method=['OPTIONS', 'POST'], name="atomRings")
def atomRings():
    """
Returns atom indexes group by SSSR rings. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @rings.mol ${BEAKER_ROOT_URL}atomRings
    curl -X POST -F "file=@rings.mol" ${BEAKER_ROOT_URL}atomRings
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return atomRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def bondRingsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return json.dumps(_bondRings(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/bondRings/<ctab>', method=['OPTIONS', 'GET'], name="bondRings")
def bondRings(ctab):
    """
Returns bond indexes group by SSSR rings. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}bondRings/$(cat rings.mol | base64 -w 0 | tr "+/" "-_")

    """

    data = base64.urlsafe_b64decode(ctab)
    return bondRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/bondRings', method=['OPTIONS', 'POST'], name="bondRings")
def bondRings():
    """
Returns bond indexes group by SSSR rings. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @rings.mol ${BEAKER_ROOT_URL}bondRings
    curl -X POST -F "file=@rings.mol" ${BEAKER_ROOT_URL}bondRings
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return bondRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def atomIsInRingView(data, params):

    kwargs = dict()
    kwargs['index'] = int(params.get('index'))
    kwargs['size'] = int(params.get('size'))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return json.dumps(_isAtomInRing(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/atomIsInRing/<ctab>/<index>/<size>', method=['OPTIONS', 'GET'], name="atomIsInRing")
def atomIsInRing(ctab, index, size):
    """
A predicate, checking if atom is in ring. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}atomIsInRing/$(cat addHs.mol | base64 -w 0 | tr "+/" "-_")/1/6

    """

    data = base64.urlsafe_b64decode(ctab)

    params = request.params
    params['index'] = index
    params['size'] = size

    return atomIsInRingView(data, params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/atomIsInRing', method=['OPTIONS', 'POST'], name="atomIsInRing")
def atomIsInRing():
    """
A predicate, checking if atom is in ring. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST -F "file=@addHs.mol" -F "index=1" -F "size=6" ${BEAKER_ROOT_URL}atomIsInRing
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return atomIsInRingView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def bondIsInRingView(data, params):

    kwargs = dict()
    kwargs['index'] = int(params.get('index'))
    kwargs['size'] = int(params.get('size'))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return json.dumps(_isBondInRing(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/bondIsInRing/<ctab>/<index>/<size>', method=['OPTIONS', 'GET'], name="bondIsInRing")
def bondIsInRing(ctab, index, size):
    """
A predicate, checking if bond is in ring. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}bondIsInRing/$(cat rings.mol | base64 -w 0 | tr "+/" "-_")/1/3

    """

    data = base64.urlsafe_b64decode(ctab)

    params = request.params
    params['index'] = index
    params['size'] = size

    return bondIsInRingView(data, params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/bondIsInRing', method=['OPTIONS', 'POST'], name="bondIsInRing")
def bondIsInRing():
    """
A predicate, checking if bond is in ring. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST -F "file=@rings.mol" -F "index=1" -F "size=3" ${BEAKER_ROOT_URL}bondIsInRing
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return bondIsInRingView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def numAtomRingsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return json.dumps(_numAtomRings(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/numAtomRings/<ctab>', method=['OPTIONS', 'GET'], name="numAtomRings")
def numAtomRings(ctab):
    """
Returns ring membership counts for every atom. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}numAtomRings/$(cat rings.mol | base64 -w 0 | tr "+/" "-_")

    """
    data = base64.urlsafe_b64decode(ctab)
    return numAtomRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/numAtomRings', method=['OPTIONS', 'POST'], name="numAtomRings")
def numAtomRings():
    """
Returns ring membership counts for every atom. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @rings.mol ${BEAKER_ROOT_URL}numAtomRings
    curl -X POST -F "file=@rings.mol" ${BEAKER_ROOT_URL}numAtomRings
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return numAtomRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def numBondRingsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return json.dumps(_numBondRings(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/numBondRings/<ctab>', method=['OPTIONS', 'GET'], name="numBondRings")
def numBondRings(ctab):
    """
Returns ring membership counts for every bond. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}numBondRings/$(cat rings.mol | base64 -w 0 | tr "+/" "-_")

    """
    data = base64.urlsafe_b64decode(ctab)
    return numBondRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/numBondRings', method=['OPTIONS', 'POST'], name="numBondRings")
def numBondRings():
    """
Returns ring membership counts for every bond. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @rings.mol ${BEAKER_ROOT_URL}numBondRings
    curl -X POST -F "file=@rings.mol" ${BEAKER_ROOT_URL}numBondRings
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return numBondRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def numRingsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return json.dumps(_numRings(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/numRings/<ctab>', method=['OPTIONS', 'GET'], name="numRings")
def numRings(ctab):
    """
Returns number of rings in molecule (NB SSSR set only). CTAB is either single molfile or SDF file.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}numRings/$(cat rings.mol | base64 -w 0 | tr "+/" "-_")
    """
    data = base64.urlsafe_b64decode(ctab)
    return numRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/numRings', method=['OPTIONS', 'POST'], name="numRings")
def numRings():
    """
Returns number of rings in molecule (NB SSSR set only). CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @rings.mol ${BEAKER_ROOT_URL}numRings
    curl -X POST -F "file=@rings.mol" ${BEAKER_ROOT_URL}numRings
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return numRingsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------


__author__ = 'mnowotka'

from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.descriptors.impl import _getNumAtoms,_getNumBonds, _getLogP, _getTPSA, _getMolWt, _getDescriptors
from bottle import request
import base64
import json

#-----------------------------------------------------------------------------------------------------------------------

def getNumAtomsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = bool(params.get('sanitize', True))
    kwargs['removeHs'] = bool(params.get('removeHs', True))
    kwargs['strictParsing'] = bool(params.get('strictParsing', True))

    return json.dumps(_getNumAtoms(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/getNumAtoms/<ctab>', method=['OPTIONS', 'GET'], name="getNumAtoms")
def getNumAtoms(ctab):
    """
Counts number of atoms of given compounds. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}getNumAtoms/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")

    """

    data = base64.urlsafe_b64decode(ctab)
    return getNumAtomsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/getNumAtoms', method=['OPTIONS', 'POST'], name="getNumAtoms")
def getNumAtoms():
    """
Counts number of atoms of given compounds. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}getNumAtoms
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}getNumAtoms
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return getNumAtomsView(data, request.params)
#-----------------------------------------------------------------------------------------------------------------------

def getNumBondsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = bool(params.get('sanitize', True))
    kwargs['removeHs'] = bool(params.get('removeHs', True))
    kwargs['strictParsing'] = bool(params.get('strictParsing', True))

    return json.dumps(_getNumBonds(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/getNumBonds/<ctab>', method=['OPTIONS', 'GET'], name="getNumBonds")
def getNumBonds(ctab):
    """
Counts number of bonds of given compounds. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}getNumBonds/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")

    """

    data = base64.urlsafe_b64decode(ctab)
    return getNumBondsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/getNumBonds', method=['OPTIONS', 'POST'], name="getNumBonds")
def getNumBonds():
    """
Counts number of bonds of given compounds. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}getNumBonds
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}getNumBonds
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return getNumBondsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def logPView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = bool(params.get('sanitize', True))
    kwargs['removeHs'] = bool(params.get('removeHs', True))
    kwargs['strictParsing'] = bool(params.get('strictParsing', True))

    return json.dumps(_getLogP(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/logP/<ctab>', method=['OPTIONS', 'GET'], name="logP")
def logP(ctab):
    """
Returns the logP value for a molecule. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}logP/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return logPView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/logP', method=['OPTIONS', 'POST'], name="logP")
def logP():
    """
Returns the logP value for a molecule. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}logP
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}logP
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return logPView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def tpsaView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = bool(params.get('sanitize', True))
    kwargs['removeHs'] = bool(params.get('removeHs', True))
    kwargs['strictParsing'] = bool(params.get('strictParsing', True))
    return json.dumps(_getTPSA(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/tpsa/<ctab>', method=['OPTIONS', 'GET'], name="tpsa")
def tpsa(ctab):
    """
Returns the TPSA value for a molecule. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}tpsa/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return tpsaView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/tpsa', method=['OPTIONS', 'POST'], name="tpsa")
def tpsa():
    """
Returns the TPSA value for a molecule. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}tpsa
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}tpsa
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return tpsaView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def molWtView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = bool(params.get('sanitize', True))
    kwargs['removeHs'] = bool(params.get('removeHs', True))
    kwargs['strictParsing'] = bool(params.get('strictParsing', True))
    return json.dumps(_getMolWt(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/molWt/<ctab>', method=['OPTIONS', 'GET'], name="molWt")
def molWt(ctab):
    """
Returns molecular weight of a compound. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}molWt/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return molWtView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/molWt', method=['OPTIONS', 'POST'], name="molWt")
def molWt():
    """
Returns molecular weight of a compound. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}molWt
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}molWt
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return molWtView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def descriptorsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = bool(params.get('sanitize', True))
    kwargs['removeHs'] = bool(params.get('removeHs', True))
    kwargs['strictParsing'] = bool(params.get('strictParsing', True))
    kwargs['ds'] = params.get('descrs')
    return json.dumps(_getDescriptors(data, **kwargs))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/descriptors/<ctab>', method=['OPTIONS', 'GET'], name="descriptors")
def descriptors(ctab):
    """
Returns descriptors of a compound. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}descriptors/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return descriptorsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/descriptors', method=['OPTIONS', 'POST'], name="descriptors")
def descriptors():
    """
Returns descriptors of a compound. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}descriptors
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}descriptors
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return descriptorsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------



# $Id$
#
#  Copyright (C) 2008-2011 Greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
from rdkit import Chem
import numpy
import math
import copy

periodicTable = Chem.GetPeriodicTable()

# ----------------------------------------------------------------------------------------------------------------------


class Font(object):
    face = 'sans'
    size = '12'
    weight = 'normal'
    name = None

    def __init__(self, face=None, size=None, name=None, weight=None):
        if face: self.face = face
        if size: self.size = size
        if name: self.name = name
        if weight: self.weight = weight


# ----------------------------------------------------------------------------------------------------------------------


class DrawingOptions(object):
    dotsPerAngstrom = 30
    useFraction = 0.90

    atomLabelFontFace = "sans"
    atomLabelFontSize = 12
    atomLabelMinFontSize = 7

    bondLineWidth = 1.2
    dblBondOffset = .20
    dblBondLengthFrac = .8

    defaultColor = (1, 0, 0)
    selectColor = (1, 0, 0)
    bgColor = (1, 1, 1)

    colorBonds = True
    noCarbonSymbols = True
    includeAtomNumbers = False
    atomNumberOffset = 0
    radicalSymbol = u'\u2219'

    dash = (4, 4)

    wedgeDashedBonds = True

    # used to adjust overall scaling for molecules that have been laid out with non-standard
    # bond lengths
    coordScale = 1.0

    elemDict = {
        1: (0.55, 0.55, 0.55),
        7: (0, 0, 1),
        8: (1, 0, 0),
        9: (.2, .8, .8),
        15: (1, .5, 0),
        16: (.8, .8, 0),
        17: (0, .8, 0),
        35: (.5, .3, .1),
        0: (.5, .5, .5),
    }

# ----------------------------------------------------------------------------------------------------------------------


class MolDrawing(object):
    atomPs = None
    canvas = None
    canvasSize = None

# ----------------------------------------------------------------------------------------------------------------------

    def __init__(self, canvas=None, drawingOptions=None):
        self.canvas = canvas
        if canvas:
            self.canvasSize = canvas.size
        self.atomPs = {}
        if drawingOptions is None:
            self.drawingOptions = DrawingOptions()
        else:
            self.drawingOptions = drawingOptions
        self.boundingBoxes = {}

        if self.drawingOptions.bgColor is not None:
            self.canvas.addCanvasPolygon(((0, 0),
                                          (canvas.size[0], 0),
                                          (canvas.size[0], canvas.size[1]),
                                          (0, canvas.size[1])),
                                         color=self.drawingOptions.bgColor,
                                         fill=True, stroke=False)

# ----------------------------------------------------------------------------------------------------------------------

    def transformPoint(self, pos):
        res = [0, 0]
        res[0] = (pos[0] + self.molTrans[0]) * self.currDotsPerAngstrom * self.drawingOptions.useFraction + \
                 self.drawingTrans[0]
        res[1] = self.canvasSize[1] - (
        (pos[1] + self.molTrans[1]) * self.currDotsPerAngstrom * self.drawingOptions.useFraction + \
        self.drawingTrans[1])
        return res

# ----------------------------------------------------------------------------------------------------------------------

    def _getBondOffset(self, p1, p2):
        # get the vector between the points:
        dx = p2[0] - p1[0]
        dy = p2[1] - p1[1]

        # figure out the angle and the perpendicular:
        ang = math.atan2(dy, dx)
        perp = ang + math.pi / 2.

        # here's the offset for the parallel bond:
        offsetX = math.cos(perp) * self.drawingOptions.dblBondOffset * self.currDotsPerAngstrom
        offsetY = math.sin(perp) * self.drawingOptions.dblBondOffset * self.currDotsPerAngstrom

        return perp, offsetX, offsetY

# ----------------------------------------------------------------------------------------------------------------------

    def _getOffsetBondPts(self, p1, p2,
                          offsetX, offsetY,
                          lenFrac=None):
        if not lenFrac:
            lenFrac = self.drawingOptions.dblBondLengthFrac

        dx = p2[0] - p1[0]
        dy = p2[1] - p1[1]
        # ----
        # now figure out where to start and end it:

        # offset the start point:
        fracP1 = p1[0] + offsetX, p1[1] + offsetY

        # now move a portion of the way along the line to the neighbor:
        frac = (1. - lenFrac) / 2
        fracP1 = fracP1[0] + dx * frac, \
                 fracP1[1] + dy * frac

        fracP2 = fracP1[0] + dx * lenFrac, \
                 fracP1[1] + dy * lenFrac
        return fracP1, fracP2

# ----------------------------------------------------------------------------------------------------------------------

    def _offsetDblBond(self, p1, p2, bond, a1, a2, conf, dir=1,
                       lenFrac=None):
        perp, offsetX, offsetY = self._getBondOffset(p1, p2)
        offsetX = offsetX * dir
        offsetY = offsetY * dir

        # if we're a ring bond, we may need to flip over to the other side:
        if bond.IsInRing():
            bondIdx = bond.GetIdx()
            a1Idx = a1.GetIdx()
            a2Idx = a2.GetIdx()
            # find a ring bond from a1 to an atom other than a2:
            for otherBond in a1.GetBonds():
                if otherBond.GetIdx() != bondIdx and \
                        otherBond.IsInRing():
                    sharedRing = False
                    for ring in self.bondRings:
                        if bondIdx in ring and otherBond.GetIdx() in ring:
                            sharedRing = True
                            break
                    if not sharedRing:
                        continue
                    a3 = otherBond.GetOtherAtom(a1)
                    if a3.GetIdx() != a2Idx:
                        p3 = self.transformPoint(conf.GetAtomPosition(a3.GetIdx()) * self.drawingOptions.coordScale)
                        dx2 = p3[0] - p1[0]
                        dy2 = p3[1] - p1[1]
                        dotP = dx2 * offsetX + dy2 * offsetY
                        if dotP < 0:
                            perp += math.pi
                            offsetX = math.cos(perp) * self.drawingOptions.dblBondOffset * self.currDotsPerAngstrom
                            offsetY = math.sin(perp) * self.drawingOptions.dblBondOffset * self.currDotsPerAngstrom

        fracP1, fracP2 = self._getOffsetBondPts(p1, p2,
                                                offsetX, offsetY,
                                                lenFrac=lenFrac)
        return fracP1, fracP2

# ----------------------------------------------------------------------------------------------------------------------

    def _getBondAttachmentCoordinates(self, p1, p2, labelSize):
        newpos = [None, None]
        if labelSize != None:
            labelSizeOffset = [labelSize[0][0] / 2 + (cmp(p2[0], p1[0]) * labelSize[0][2]), labelSize[0][1] / 2]
            if p1[1] == p2[1]:
                newpos[0] = p1[0] + cmp(p2[0], p1[0]) * labelSizeOffset[0]
            else:
                if abs(labelSizeOffset[1] * (p2[0] - p1[0]) / (p2[1] - p1[1])) < labelSizeOffset[0]:
                    newpos[0] = p1[0] + cmp(p2[0], p1[0]) * abs(labelSizeOffset[1] * (p2[0] - p1[0]) / (p2[1] - p1[1]))
                else:
                    newpos[0] = p1[0] + cmp(p2[0], p1[0]) * labelSizeOffset[0]
            if p1[0] == p2[0]:
                newpos[1] = p1[1] + cmp(p2[1], p1[1]) * labelSizeOffset[1]
            else:
                if abs(labelSizeOffset[0] * (p1[1] - p2[1]) / (p2[0] - p1[0])) < labelSizeOffset[1]:
                    newpos[1] = p1[1] + cmp(p2[1], p1[1]) * abs(labelSizeOffset[0] * (p1[1] - p2[1]) / (p2[0] - p1[0]))
                else:
                    newpos[1] = p1[1] + cmp(p2[1], p1[1]) * labelSizeOffset[1]
        else:
            newpos = copy.deepcopy(p1)
        return newpos

# ----------------------------------------------------------------------------------------------------------------------

    def _drawWedgedBond(self, bond, pos, nbrPos,
                        width=None, color=None,
                        dash=None):
        if width is None:
            width = self.drawingOptions.bondLineWidth
        if color is None:
            color = self.drawingOptions.defaultColor
        perp, offsetX, offsetY = self._getBondOffset(pos, nbrPos)
        offsetX *= .75
        offsetY *= .75
        poly = ((pos[0], pos[1]),
                (nbrPos[0] + offsetX, nbrPos[1] + offsetY),
                (nbrPos[0] - offsetX, nbrPos[1] - offsetY))
        # canvas.drawPolygon(poly,edgeColor=color,edgeWidth=1,fillColor=color,closed=1)
        if not dash:
            self.canvas.addCanvasPolygon(poly, color=color)
        elif self.drawingOptions.wedgeDashedBonds and self.canvas.addCanvasDashedWedge:
            self.canvas.addCanvasDashedWedge(poly[0], poly[1], poly[2], color=color)
        else:
            self.canvas.addCanvasLine(pos, nbrPos, linewidth=width * 2, color=color,
                                      dashes=dash)

# ----------------------------------------------------------------------------------------------------------------------

    def _drawBond(self, bond, atom, nbr, pos, nbrPos, conf,
                  width=None, color=None, color2=None, labelSize1=None, labelSize2=None):
        if width is None:
            width = self.drawingOptions.bondLineWidth
        if color is None:
            color = self.drawingOptions.defaultColor
        p1_raw = copy.deepcopy(pos)
        p2_raw = copy.deepcopy(nbrPos)
        newpos = self._getBondAttachmentCoordinates(p1_raw, p2_raw, labelSize1)
        newnbrPos = self._getBondAttachmentCoordinates(p2_raw, p1_raw, labelSize2)
        bType = bond.GetBondType()
        if bType == Chem.BondType.SINGLE:
            bDir = bond.GetBondDir()
            if bDir in (Chem.BondDir.BEGINWEDGE, Chem.BondDir.BEGINDASH):
                # if the bond is "backwards", change the drawing direction:
                if bond.GetBeginAtom().GetChiralTag() in (Chem.ChiralType.CHI_TETRAHEDRAL_CW,
                                                          Chem.ChiralType.CHI_TETRAHEDRAL_CCW):
                    p1, p2 = newpos, newnbrPos
                    wcolor = color
                else:
                    p2, p1 = newpos, newnbrPos
                    if color2 is not None:
                        wcolor = color2
                    else:
                        wcolor = self.drawingOptions.defaultColor
                if bDir == Chem.BondDir.BEGINWEDGE:
                    self._drawWedgedBond(bond, p1, p2, color=wcolor, width=width)
                elif bDir == Chem.BondDir.BEGINDASH:
                    self._drawWedgedBond(bond, p1, p2, color=wcolor, width=width,
                                         dash=self.drawingOptions.dash)

            else:
                self.canvas.addCanvasLine(newpos, newnbrPos, linewidth=width, color=color, color2=color2)
        elif bType == Chem.BondType.DOUBLE:
            if bond.IsInRing() or (atom.GetDegree() != 1 and bond.GetOtherAtom(atom).GetDegree() != 1):
                self.canvas.addCanvasLine(newpos, newnbrPos, linewidth=width, color=color, color2=color2)
                fp1, fp2 = self._offsetDblBond(newpos, newnbrPos, bond, atom, nbr, conf)
                self.canvas.addCanvasLine(fp1, fp2, linewidth=width, color=color, color2=color2)
            else:
                fp1, fp2 = self._offsetDblBond(newpos, newnbrPos, bond, atom, nbr, conf, dir=.5,
                                               lenFrac=1.0)
                self.canvas.addCanvasLine(fp1, fp2, linewidth=width, color=color, color2=color2)
                fp1, fp2 = self._offsetDblBond(newpos, newnbrPos, bond, atom, nbr, conf, dir=-.5,
                                               lenFrac=1.0)
                self.canvas.addCanvasLine(fp1, fp2, linewidth=width, color=color, color2=color2)
        elif bType == Chem.BondType.AROMATIC:
            self.canvas.addCanvasLine(newpos, newnbrPos, linewidth=width, color=color, color2=color2)
            fp1, fp2 = self._offsetDblBond(newpos, newnbrPos, bond, atom, nbr, conf)
            self.canvas.addCanvasLine(fp1, fp2, linewidth=width, color=color, color2=color2,
                                      dash=self.drawingOptions.dash)
        elif bType == Chem.BondType.TRIPLE:
            self.canvas.addCanvasLine(newpos, newnbrPos, linewidth=width, color=color, color2=color2)
            fp1, fp2 = self._offsetDblBond(newpos, newnbrPos, bond, atom, nbr, conf)
            self.canvas.addCanvasLine(fp1, fp2, linewidth=width, color=color, color2=color2)
            fp1, fp2 = self._offsetDblBond(newpos, newnbrPos, bond, atom, nbr, conf, dir=-1)
            self.canvas.addCanvasLine(fp1, fp2, linewidth=width, color=color, color2=color2)
        else:
            self.canvas.addCanvasLine(newpos, newnbrPos, linewidth=width, color=color, color2=color2,
                                      dash=(1, 2))

# ----------------------------------------------------------------------------------------------------------------------

    def scaleAndCenter(self, mol, conf, coordCenter=False, canvasSize=None, ignoreHs=False):
        if canvasSize is None:
            canvasSize = self.canvasSize
        xAccum = 0
        yAccum = 0
        minX = 1e8
        minY = 1e8
        maxX = -1e8
        maxY = -1e8

        nAts = mol.GetNumAtoms()
        for i in range(nAts):
            if ignoreHs and mol.GetAtomWithIdx(i).GetAtomicNum() == 1: continue
            pos = conf.GetAtomPosition(i) * self.drawingOptions.coordScale
            xAccum += pos[0]
            yAccum += pos[1]
            minX = min(minX, pos[0])
            minY = min(minY, pos[1])
            maxX = max(maxX, pos[0])
            maxY = max(maxY, pos[1])

        dx = abs(maxX - minX)
        dy = abs(maxY - minY)
        xSize = dx * self.currDotsPerAngstrom
        ySize = dy * self.currDotsPerAngstrom

        if coordCenter:
            molTrans = -xAccum / nAts, -yAccum / nAts
        else:
            molTrans = -(minX + (maxX - minX) / 2), -(minY + (maxY - minY) / 2)
        self.molTrans = molTrans

        if xSize >= .95 * canvasSize[0]:
            scale = .9 * canvasSize[0] / xSize
            xSize *= scale
            ySize *= scale
            self.currDotsPerAngstrom *= scale
            self.currAtomLabelFontSize = max(self.currAtomLabelFontSize * scale,
                                             self.drawingOptions.atomLabelMinFontSize)
        if ySize >= .95 * canvasSize[1]:
            scale = .9 * canvasSize[1] / ySize
            xSize *= scale
            ySize *= scale
            self.currDotsPerAngstrom *= scale
            self.currAtomLabelFontSize = max(self.currAtomLabelFontSize * scale,
                                             self.drawingOptions.atomLabelMinFontSize)
        drawingTrans = canvasSize[0] / 2, canvasSize[1] / 2
        self.drawingTrans = drawingTrans

# ----------------------------------------------------------------------------------------------------------------------

    def _drawLabel(self, label, pos, baseOffset, font, color=None, **kwargs):
        if color is None:
            color = self.drawingOptions.defaultColor
        x1 = pos[0]
        y1 = pos[1]
        labelP = x1, y1
        labelSize = self.canvas.addCanvasText(label, (x1, y1, baseOffset), font, color, **kwargs)
        return labelSize

# ----------------------------------------------------------------------------------------------------------------------

    def AddMol(self, mol, centerIt=True, molTrans=None, drawingTrans=None,
               highlightAtoms=[], confId=-1, flagCloseContactsDist=2,
               highlightMap=None, ignoreHs=False, highlightBonds=[], **kwargs):
        """Set the molecule to be drawn.
    
        Parameters:
          hightlightAtoms -- list of atoms to highlight (default [])
          highlightMap -- dictionary of (atom, color) pairs (default None)
    
        Notes:
          - specifying centerIt will cause molTrans and drawingTrans to be ignored
        """
        conf = mol.GetConformer(confId)
        if 'coordScale' in kwargs:
            self.drawingOptions.coordScale = kwargs['coordScale']

        self.currDotsPerAngstrom = self.drawingOptions.dotsPerAngstrom
        self.currAtomLabelFontSize = self.drawingOptions.atomLabelFontSize
        if centerIt:
            self.scaleAndCenter(mol, conf, ignoreHs=ignoreHs)
        else:
            if molTrans is None:
                molTrans = (0, 0)
            self.molTrans = molTrans
            if drawingTrans is None:
                drawingTrans = (0, 0)
            self.drawingTrans = drawingTrans

        font = Font(face=self.drawingOptions.atomLabelFontFace, size=self.currAtomLabelFontSize)

        obds = None
        if not mol.HasProp('_drawingBondsWedged'):
            # this is going to modify the molecule, get ready to undo that
            obds = [x.GetBondDir() for x in mol.GetBonds()]
            Chem.WedgeMolBonds(mol, conf)

        includeAtomNumbers = kwargs.get('includeAtomNumbers', self.drawingOptions.includeAtomNumbers)
        self.atomPs[mol] = {}
        self.boundingBoxes[mol] = [0] * 4
        self.activeMol = mol
        self.bondRings = mol.GetRingInfo().BondRings()
        labelSizes = {}
        for atom in mol.GetAtoms():
            labelSizes[atom.GetIdx()] = None
            if ignoreHs and atom.GetAtomicNum() == 1:
                drawAtom = False
            else:
                drawAtom = True
            idx = atom.GetIdx()
            pos = self.atomPs[mol].get(idx, None)
            if pos is None:
                pos = self.transformPoint(conf.GetAtomPosition(idx) * self.drawingOptions.coordScale)
                self.atomPs[mol][idx] = pos
                if drawAtom:
                    self.boundingBoxes[mol][0] = min(self.boundingBoxes[mol][0], pos[0])
                    self.boundingBoxes[mol][1] = min(self.boundingBoxes[mol][1], pos[1])
                    self.boundingBoxes[mol][2] = max(self.boundingBoxes[mol][2], pos[0])
                    self.boundingBoxes[mol][3] = max(self.boundingBoxes[mol][3], pos[1])

            if not drawAtom: continue
            nbrSum = [0, 0]
            for bond in atom.GetBonds():
                nbr = bond.GetOtherAtom(atom)
                if ignoreHs and nbr.GetAtomicNum() == 1: continue
                nbrIdx = nbr.GetIdx()
                if nbrIdx > idx:
                    nbrPos = self.atomPs[mol].get(nbrIdx, None)
                    if nbrPos is None:
                        nbrPos = self.transformPoint(conf.GetAtomPosition(nbrIdx) * self.drawingOptions.coordScale)
                        self.atomPs[mol][nbrIdx] = nbrPos
                        self.boundingBoxes[mol][0] = min(self.boundingBoxes[mol][0], nbrPos[0])
                        self.boundingBoxes[mol][1] = min(self.boundingBoxes[mol][1], nbrPos[1])
                        self.boundingBoxes[mol][2] = max(self.boundingBoxes[mol][2], nbrPos[0])
                        self.boundingBoxes[mol][3] = max(self.boundingBoxes[mol][3], nbrPos[1])

                else:
                    nbrPos = self.atomPs[mol][nbrIdx]
                nbrSum[0] += nbrPos[0] - pos[0]
                nbrSum[1] += nbrPos[1] - pos[1]

            iso = atom.GetIsotope()

            labelIt = not self.drawingOptions.noCarbonSymbols or \
                      atom.GetAtomicNum() != 6 or \
                      atom.GetFormalCharge() != 0 or \
                      atom.GetNumRadicalElectrons() or \
                      includeAtomNumbers or \
                      iso or \
                      atom.HasProp('molAtomMapNumber') or \
                      atom.GetDegree() == 0
            orient = ''
            if labelIt:
                baseOffset = 0
                if includeAtomNumbers:
                    symbol = str(atom.GetIdx())
                    symbolLength = len(symbol)
                else:
                    base = atom.GetSymbol()
                    symbolLength = len(base)
                    nHs = atom.GetTotalNumHs()
                    if nHs > 0:
                        if nHs > 1:
                            hs = 'H<sub>%d</sub>' % nHs
                            symbolLength += 1 + len(str(nHs))
                        else:
                            hs = 'H'
                            symbolLength += 1
                    else:
                        hs = ''
                    chg = atom.GetFormalCharge()
                    if chg != 0:
                        if chg == 1:
                            chg = '+'
                        elif chg == -1:
                            chg = '-'
                        elif chg > 1:
                            chg = '+%d' % chg
                        elif chg < -1:
                            chg = '-%d' % chg
                        symbolLength += len(chg)
                    else:
                        chg = ''
                    if chg:
                        chg = '<sup>%s</sup>' % chg

                    if atom.GetNumRadicalElectrons():
                        rad = self.drawingOptions.radicalSymbol * atom.GetNumRadicalElectrons()
                        rad = '<sup>%s</sup>' % rad
                        symbolLength += atom.GetNumRadicalElectrons()
                    else:
                        rad = ''

                    isotope = ''
                    isotopeLength = 0
                    if iso:
                        isotope = '<sup>%d</sup>' % atom.GetIsotope()
                        isotopeLength = len(str(atom.GetIsotope()))
                        symbolLength += isotopeLength
                    mapNum = ''
                    mapNumLength = 0
                    if atom.HasProp('molAtomMapNumber'):
                        mapNum = ':' + atom.GetProp('molAtomMapNumber')
                        mapNumLength = 1 + len(str(atom.GetProp('molAtomMapNumber')))
                        symbolLength += mapNumLength
                    deg = atom.GetDegree()
                    # This should be done in a better way in the future:
                    # 'baseOffset' should be determined by getting the size of 'isotope' and the size of 'base', or the size of 'mapNum' and the size of 'base'
                    # (depending on 'deg' and 'nbrSum[0]') in order to determine the exact position of the base
                    if deg == 0:
                        if periodicTable.GetElementSymbol(atom.GetAtomicNum()) in (
                        'O', 'S', 'Se', 'Te', 'F', 'Cl', 'Br', 'I', 'At'):
                            symbol = '%s%s%s%s%s%s' % (hs, isotope, base, chg, rad, mapNum)
                        else:
                            symbol = '%s%s%s%s%s%s' % (isotope, base, hs, chg, rad, mapNum)
                    elif deg > 1 or nbrSum[0] < 1:
                        symbol = '%s%s%s%s%s%s' % (isotope, base, hs, chg, rad, mapNum)
                        baseOffset = 0.5 - (isotopeLength + len(base) / 2.) / symbolLength
                    else:
                        symbol = '%s%s%s%s%s%s' % (rad, chg, hs, isotope, base, mapNum)
                        baseOffset = -0.5 + (mapNumLength + len(base) / 2.) / symbolLength
                    if deg == 1:
                        if abs(nbrSum[1]) > 1:
                            islope = nbrSum[0] / abs(nbrSum[1])
                        else:
                            islope = nbrSum[0]
                        if abs(islope) > .3:
                            if islope > 0:
                                orient = 'W'
                            else:
                                orient = 'E'
                        elif abs(nbrSum[1]) > 10:
                            if nbrSum[1] > 0:
                                orient = 'N'
                            else:
                                orient = 'S'
                    else:
                        orient = 'C'
                if highlightMap and idx in highlightMap:
                    color = highlightMap[idx]
                elif highlightAtoms and idx in highlightAtoms:
                    color = self.drawingOptions.selectColor
                else:
                    color = self.drawingOptions.elemDict.get(atom.GetAtomicNum(), (0, 0, 0))
                labelSize = self._drawLabel(symbol, pos, baseOffset, font, color=color, orientation=orient)
                labelSizes[atom.GetIdx()] = [labelSize, orient]

        for bond in mol.GetBonds():
            atom, idx = bond.GetBeginAtom(), bond.GetBeginAtomIdx()
            nbr, nbrIdx = bond.GetEndAtom(), bond.GetEndAtomIdx()
            pos = self.atomPs[mol].get(idx, None)
            nbrPos = self.atomPs[mol].get(nbrIdx, None)
            if highlightBonds and bond.GetIdx() in highlightBonds:
                width = 2.0 * self.drawingOptions.bondLineWidth
                color = self.drawingOptions.selectColor
                color2 = self.drawingOptions.selectColor
            elif highlightAtoms and idx in highlightAtoms and nbrIdx in highlightAtoms:
                width = 2.0 * self.drawingOptions.bondLineWidth
                color = self.drawingOptions.selectColor
                color2 = self.drawingOptions.selectColor
            elif highlightMap is not None and idx in highlightMap and nbrIdx in highlightMap:
                width = 2.0 * self.drawingOptions.bondLineWidth
                color = highlightMap[idx]
                color2 = highlightMap[nbrIdx]
            else:
                width = self.drawingOptions.bondLineWidth
                if self.drawingOptions.colorBonds:
                    color = self.drawingOptions.elemDict.get(atom.GetAtomicNum(), (0, 0, 0))
                    color2 = self.drawingOptions.elemDict.get(nbr.GetAtomicNum(), (0, 0, 0))
                else:
                    color = self.drawingOptions.defaultColor
                    color2 = color
            self._drawBond(bond, atom, nbr, pos, nbrPos, conf,
                           color=color, width=width, color2=color2, labelSize1=labelSizes[idx],
                           labelSize2=labelSizes[nbrIdx])

        # if we modified the bond wedging state, undo those changes now
        if obds:
            for i, d in enumerate(obds):
                mol.GetBondWithIdx(i).SetBondDir(d)

        if flagCloseContactsDist > 0:
            tol = flagCloseContactsDist * flagCloseContactsDist
            for i, atomi in enumerate(mol.GetAtoms()):
                pi = numpy.array(self.atomPs[mol][i])
                for j in range(i + 1, mol.GetNumAtoms()):
                    pj = numpy.array(self.atomPs[mol][j])
                    d = pj - pi
                    dist2 = d[0] * d[0] + d[1] * d[1]
                    if dist2 <= tol:
                        self.canvas.addCanvasPolygon(((pi[0] - 2 * flagCloseContactsDist,
                                                       pi[1] - 2 * flagCloseContactsDist),
                                                      (pi[0] + 2 * flagCloseContactsDist,
                                                       pi[1] - 2 * flagCloseContactsDist),
                                                      (pi[0] + 2 * flagCloseContactsDist,
                                                       pi[1] + 2 * flagCloseContactsDist),
                                                      (pi[0] - 2 * flagCloseContactsDist,
                                                       pi[1] + 2 * flagCloseContactsDist)),
                                                     color=(1., 0, 0),
                                                     fill=False, stroke=True)

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

from chembl_beaker.beaker import app
from bottle import request, response
from chembl_beaker.beaker.core_apps.calculations.impl import _kekulize, _sanitize, _addHs
from chembl_beaker.beaker.core_apps.calculations.impl import _removeHs, _sssr, _symmsssr
from chembl_beaker.beaker.core_apps.calculations.impl import _align
from chembl_beaker.beaker.utils.io import _parseFlag
import base64
import json

# ----------------------------------------------------------------------------------------------------------------------


def alignView(data, template, params):
    force = _parseFlag(params.get('force', False))
    ret = _align(data, template, force)
    if ret:
        return ret
    response.status = 404
    response.body = 'No template found in any of structures given.'

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/align/<template>/<ctab>', method=['OPTIONS', 'GET'], name="align")
def align(template, ctab):
    """
Generate a depiction for a molecules given as CTAB where a piece of the molecule is constrained to have the same 
coordinates as a reference molfile. TEMPLATE is urlsafe_base64 encoded string containing a single molfile.
CTAB is urlsafe_base64 encoded string containing concatenation of multiple molfiles to be aligned.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}align/$(cat pattern.mol | base64 -w 0 | tr "+/" "-_")/$(cat mcs.sdf | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}align/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")/$(cat chembl_1999443.mol | base64 -w 0 | tr "+/" "-_")?force=true
    """

    data = base64.urlsafe_b64decode(ctab)
    t = base64.urlsafe_b64decode(template)
    return alignView(data, t, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/align', method=['OPTIONS', 'POST'], name="align")
def align():
    """
Generate a depiction for a molecules. Expects either two CTABS - one with a template fragment and another one with 
molecules to be aligned or a single CTAB with the first molecule being a template and others so be aligned.
cURL examples:

    curl -X POST -F "template=@pattern.mol" -F "ctab=@mcs.sdf" ${BEAKER_ROOT_URL}align
    curl -X POST -F "template=@aspirin.mol" -F "ctab=@chembl_1999443.mol"  -F "force=true" ${BEAKER_ROOT_URL}align
    curl -X POST -F "ctab=@pattern_and_mcs.sdf" ${BEAKER_ROOT_URL}align
    """

    number_of_files = len(request.files)

    if number_of_files == 1:
        ctab = request.files['ctab'].file.read()
        return alignView(ctab, None, request.params)

    elif number_of_files == 2:
        template = request.files['template'].file.read()
        ctab = request.files['ctab'].file.read()
        return alignView(ctab, template, request.params)

    response.status = 400
    response.body = 'This method expects one ot two ctabs.'
    print 'this should be error 400'
    return response

# ----------------------------------------------------------------------------------------------------------------------


def kekulizeView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return _kekulize(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/kekulize/<ctab>', method=['OPTIONS', 'GET'], name="kekulize")
def kekulize(ctab):
    """
Performs kekulisation on input compounds. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}kekulize/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return kekulizeView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/kekulize', method=['OPTIONS', 'POST'], name="kekulize")
def kekulize():
    """
Performs kekulisation on input compounds. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}kekulize
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}kekulize
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return kekulizeView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def sanitizeView(data, params):

    sanitizeOps = params.get('sanitizeOps')
    if sanitizeOps is None:
        return _sanitize(data)
    return _sanitize(data, int(sanitizeOps))

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/sanitize/<ctab>', method=['OPTIONS', 'GET'], name="sanitize")
def sanitize(ctab):
    """
Kekulize, check valencies, set aromaticity, conjugation and hybridization. CTAB is urlsafe_base64 encoded string
containing single molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}sanitize/$(cat aromatic.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return sanitizeView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/sanitize', method=['OPTIONS', 'POST'], name="sanitize")
def sanitize():
    """
Kekulize, check valencies, set aromaticity, conjugation and hybridization. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aromatic.mol ${BEAKER_ROOT_URL}sanitize
    curl -X POST -F "file=@aromatic.mol" ${BEAKER_ROOT_URL}sanitize
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return sanitizeView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def symmSSSRView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return json.dumps(_symmsssr(data, **kwargs))

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/symmSSSR/<ctab>', method=['OPTIONS', 'GET'], name="symmSSSR")
def symmSSSR(ctab):
    """
Get a symmetrized SSSR for a molecule. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}symmSSSR/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return symmSSSRView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/symmSSSR', method=['OPTIONS', 'POST'], name="symmSSSR")
def symmSSSR():
    """
Get a symmetrized SSSR for a molecule. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}symmSSSR
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}symmSSSR
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return symmSSSRView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def sssrView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return json.dumps(_sssr(data, **kwargs))

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/sssr/<ctab>', method=['OPTIONS', 'GET'], name="sssr")
def sssr(ctab):
    """
Get the smallest set of simple rings for a molecule. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}sssr/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return sssrView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/sssr', method=['OPTIONS', 'POST'], name="sssr")
def sssr():
    """
Get the smallest set of simple rings for a molecule. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}sssr
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}sssr
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return sssrView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def addHsView(data, params):

    kwargs = dict()
    kwargs['explicitOnly'] = _parseFlag(params.get('explicitOnly', False))
    kwargs['addCoords'] = _parseFlag(params.get('addCoords', False))

    return _addHs(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/addHs/<ctab>', method=['OPTIONS', 'GET'], name="addHs")
def addHs(ctab):
    """
Adds hydrogens to the graph of a molecule. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}addHs/$(cat addHs.mol | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}addHs/$(cat addHs.mol | base64 -w 0 | tr "+/" "-_")?addCoords=1
    """

    data = base64.urlsafe_b64decode(ctab)
    return addHsView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/addHs', method=['OPTIONS', 'POST'], name="addHs")
def addHs():
    """
Adds hydrogens to the graph of a molecule. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @addHs.mol ${BEAKER_ROOT_URL}addHs
    curl -X POST -F "file=@addHs.mol" -F "addCoords=1" ${BEAKER_ROOT_URL}addHs
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return addHsView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def removeHsView(data, params):

    kwargs = dict()
    kwargs['implicitOnly'] = _parseFlag(params.get('implicitOnly', False))
    return _removeHs(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/removeHs/<ctab>', method=['OPTIONS', 'GET'], name="removeHs")
def removeHs(ctab):
    """
Removes any hydrogens from the graph of a molecule. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}removeHs/$(cat removeHs.mol | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}removeHs/$(cat removeHs.mol | base64 -w 0 | tr "+/" "-_")?implicitOnly=1
    """

    data = base64.urlsafe_b64decode(ctab)
    return removeHsView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/removeHs', method=['OPTIONS', 'POST'], name="removeHs")
def removeHs():
    """
Removes any hydrogens from the graph of a molecule. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @removeHs.mol ${BEAKER_ROOT_URL}removeHs
    curl -X POST -F "file=@removeHs.mol" -F "implicitOnly=1" ${BEAKER_ROOT_URL}removeHs
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return removeHsView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------
